import React, {Component, Fragment} from 'react';
import _ from 'lodash';
import {connect} from 'react-redux'
import {withRouter, Route, Switch, Redirect} from 'react-router-dom'
import {Query, withApollo} from 'react-apollo'
import ContentWrapper from './components/ContentWrapper'
import TabsHeader from './components/tabs/TabsHeader'
import {GET_CLUSTERS_AND_ACCOUNT_INFO_QUERY} from './queries'
import Discover from './components/discover/Discover'
import WizardStarter from './components/tierClusterWizard/WizardStarter'
import WizardHeader from './components/wizard/WizardHeader'
import Welcome from './components/discover/Welcome'
import LoadingIcon from './components/widgets/LoadingIcon'
import MoreInfo, {InfoHeader} from "./components/tabs/MoreInfo";
import Isvg from "react-inlinesvg";
import {STATUS_SUCCESS} from "./actions";
import ClustersDashboard, {ClustersDashboardHeader} from "./components/tabs/ClustersDashboardNew";
import TabsFooter from './components/tabs/TabsFooter'
import Intercom from "./components/intercom/Intercom";
import Oops from './components/Oops';
import LicenseSettings, {LicenseHeader} from './components/tabs/license/LicenseSettings'
import './app.scss';
import {SHOW_TRIAL_INFO} from "./utils/consts";

const LoginRedirectToPath = () => {
    window.auth.loginRedirect({}, {
        service: "cloudtiering",
        startOnSignup: false
    }, {pathname: window.location.pathname});
    return <div></div>
};
const AuthenticatedRoute = props => props.auth.authenticated ? <Route {...props}/> : <LoginRedirectToPath/>;

const whereTo = (error, clusters, accountInfo) => {
    if (SHOW_TRIAL_INFO && accountInfo.daysLeftForTrial < 0) return "/license";
    else if (!error && clusters.length === 0) return "/welcome";
    else return "/clusters";
};

const AppRoutes = ({auth, clusters, accountInfo, error}) => {
    return (
        <Switch>
            <AuthenticatedRoute auth={auth} path="/clusters" exact
                                render={() => <ContentWrapper content={ClustersDashboard}
                                                              componentHeader={TabsHeader}
                                                              secondaryHeader={ClustersDashboardHeader}
                                                              footer={TabsFooter} data={{accountInfo}}/>}/>
            <AuthenticatedRoute auth={auth} path="/discover" exact render={() => <Discover/>}/>
            <AuthenticatedRoute auth={auth} path="/welcome" exact
                                render={() => <ContentWrapper content={Welcome} noNavigationToDashboard={true}/>}/>
            <AuthenticatedRoute auth={auth} path="/tier/:clusterId/volumes"
                                render={() => <ContentWrapper content={WizardStarter} nakedHeader={true}
                                                              secondaryHeader={WizardHeader} bodyClass="wizard"
                                                              data={{clusters, wizardType: "tier"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/retier/:clusterId/volumes"
                                render={() => <ContentWrapper content={WizardStarter} nakedHeader={true}
                                                              secondaryHeader={WizardHeader} bodyClass="wizard"
                                                              data={{clusters, wizardType: "retier"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/:clusterId/info"
                                render={() => <ContentWrapper content={MoreInfo}
                                                              secondaryHeader={InfoHeader}
                                                              data={{clusters}}/>}/>
            <AuthenticatedRoute auth={auth} path="/errorPage" exact render={() => <ContentWrapper content={Oops}
                                                                                                  nakedHeader={true}
                                                                                                  data={{error}}/>}/>
            data={error}/>}/>
            <AuthenticatedRoute auth={auth} path="/license" exact
                                render={() => <ContentWrapper content={LicenseSettings}
                                                              componentHeader={TabsHeader}
                                                              secondaryHeader={LicenseHeader}
                                                              footer={TabsFooter} data={{accountInfo, clusters}}/>}/>

            <Redirect to={whereTo(error, clusters, accountInfo)}/>
        </Switch>
    );
};


class App extends Component {
    componentDidMount() {
        const {error, history, accountInfo} = this.props;
        console.log(`%c Account Info ${JSON.stringify(accountInfo)}`, 'font-weight:bold;color:blue');
        if (error) {
            console.log(`%c APP mount - got error ${JSON.stringify(error)} location: ${history.location.pathname} going to errorPage`, 'font-weight:bold; color:red');
            history.replace('/errorPage');
        }
        else if (SHOW_TRIAL_INFO && accountInfo && accountInfo.daysLeftForTrial <= 0 && accountInfo.subscriptionStatus !== "subscribed" && history.location.pathname.indexOf('/license') === -1) {
            console.log(`%c APP update - no more days for trial ${JSON.stringify(accountInfo.daysLeftForTrial)} forcing license page`, 'font-weight:bold; color:red');
            history.replace('/license');
        }
    }

    componentDidUpdate(prevProps) {
        const {error, history, accountInfo} = prevProps;
        if (error && history.location.pathname.indexOf('/errorPage') === -1) {
            console.log(`%c APP update - got error ${JSON.stringify(error)} location: ${history.location.pathname} going to errorPage`, 'font-weight:bold; color:red');
            history.replace('/errorPage');
        }
        else if (SHOW_TRIAL_INFO && accountInfo && accountInfo.daysLeftForTrial <= 0 && accountInfo.subscriptionStatus !== "subscribed" && history.location.pathname.indexOf('/license') === -1) {
            console.log(`%c APP update - no more days for trial ${JSON.stringify(accountInfo.daysLeftForTrial)} forcing license page`, 'font-weight:bold; color:red');
            history.replace('/license');
        }
    }

    render() {
        const {auth, clusters, accountInfo, error} = this.props;
        document.body.className = "with-sso"; //to add the widget
        return (
            <Fragment>
                <Intercom/>
                <AppRoutes auth={auth} clusters={clusters} accountInfo={accountInfo} error={error}/>
            </Fragment>
        );
    }
}


const WithRouterApp = withRouter(App);

const mapStateToProps = state => {
    return {
        auth: state.auth.authenticated ? state.auth : null,
        routing: state.router
    };
};


class InitialQuery extends Component {
    async componentDidMount() {
        console.log(`%c APP clearing store due to first time navigation`, 'font-weight:bold; color:purple');
        await this.props.client.clearStore();
    }

    getClustersUX(auth,clusters, accountInfo, errorMessage) {
        return (
            <Fragment>
                {(clusters || errorMessage) && <WithRouterApp auth={auth} clusters={clusters} accountInfo={accountInfo} error={errorMessage}/>}
                {!clusters && ! errorMessage && <LoadingIcon text={"Clusters"} icon={<Isvg src={require('./assets/svgs/review-cluster.svg')}/>}/>}
            </Fragment>
        )
    }

    render() {
        const {auth} = this.props;
        if (!auth || auth.status !== STATUS_SUCCESS) {
            return <div></div>;
        } else {
            return (
                <Query query={GET_CLUSTERS_AND_ACCOUNT_INFO_QUERY} fetchPolicy="network-only">
                    {({loading, error, data}) => {
                        if (loading)
                            return (<LoadingIcon text={"Cloud Tiering"} icon={<Isvg
                                src={require('./assets/svgs/logo.svg')}/>}/>);
                        const isAuthenticationError = (error) =>  (_.get(error, 'networkError.result.errors[0].extensions.code') === 401 || (_.get(error, 'graphQLErrors[0].extensions.code') === 401));
                        const errorMessage = error ? (isAuthenticationError(error) ? null : error.message) : null;
                        const clusters = !error && data.clusters ? data.clusters : null;
                        if (!clusters) console.log(`APP unable to get clusters error ${JSON.stringify(error)} data ${JSON.stringify(data)}`);
                        const accountInfo = SHOW_TRIAL_INFO ? data.accountInfo : null;
                        return this.getClustersUX(auth, clusters, accountInfo, errorMessage);
                    }}
                </Query>
            )
        }
    }
};

export default connect(mapStateToProps)(withApollo(InitialQuery));