import React, {Component, Fragment} from "react"
import SearchWidget from "./../widgets/SearchWidget"
import ItemCountWidget from "./../widgets/ItemCountWidget"
import SelectedCountWidget from "./../widgets/SelectedCountWidget"
import {defaultFilter, defaultFilterWithKeys} from "../../utils/utils";
import _ from "lodash";
import {CSSTransition, TransitionGroup} from "react-transition-group";
import classNames from "classnames";
import FilterWidget from '../widgets/FilterWidget'
import Isvg from "react-inlinesvg";

require("./searchableSelectableTable.scss");


export const HEADER_TYPES = {
    "ALL": "all",
    "FILTER": "filter",
    "TD": "td"
};

export class SelectableTable extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {sortAscending: true};
    }

    applyFilter(filterMethod, filterName, filters) {
        const filtersCount = _.reduce(filters, (count, value) => count + (value.checked ? 1 : 0), 0);
        this.setState({[filterName]: {filterOpen: false, filtersCount}});
        filterMethod(filters, this.state.sort);
    }

    handleSort(sortMethod, column) {
        //if columns was not sorted yet, we start with ascending
        const ascending = this.state.sort && this.state.sort.column === column ? !this.state.sort.ascending : true;
        this.setState({sort: {column, ascending}});
        sortMethod(column, ascending)
    }

    render() {
        const {Table, data, selectionOptions, fixedHeader} = this.props;

        const headerElement = Table.getTableHeader ? Table.getTableHeader().map((cell, index) => {
            const type = cell.type ? cell.type : HEADER_TYPES.TD;
            const filtersCountFromState = cell.filterOptions ? _.get(this.state, `${cell.filterOptions.filterName}.filtersCount`, undefined) : undefined;
            const filtersCount = cell.filterOptions ? (filtersCountFromState ? filtersCountFromState : _.reduce(cell.filterOptions.filterValues, (count, value) => count + (value.checked ? 1 : 0), 0)) : undefined;
            return (
                <Fragment key={index}>
                    {type === HEADER_TYPES.ALL &&
                    <td key={index}
                        className={classNames(cell.className, "all-selector", {selected: cell.allOptions.allSelected})}
                        onClick={() => cell.allOptions.toggleAll()}>
                        <div>
                            <Isvg cacheGetRequests={true}
                                  src={require("./../../assets/svgs/checkmark-transparent-circle-icon.svg")}></Isvg>
                        </div>
                    </td>}
                    {type === HEADER_TYPES.FILTER &&
                    <td key={index} className={`${cell.className} filter-header`}>
                        <button type="button" onClick={() => this.setState({[cell.filterOptions.filterName] : {filterOpen: true, filtersCount}})}>
                            {cell.text} {`[${filtersCount}]`}
                            <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/filter.svg")}></Isvg>
                        </button>
                        {cell.filterOptions && this.state[cell.filterOptions.filterName] && this.state[cell.filterOptions.filterName].filterOpen &&
                        <FilterWidget filterValues={cell.filterOptions.filterValues}
                                      close={() => this.setState({[cell.filterOptions.filterName]: {filterOpen: false, filtersCount}})}
                                      apply={this.applyFilter.bind(this, cell.filterOptions.filter, cell.filterOptions.filterName)}
                                      fixed={this.state.headerFixed}/>}
                    </td>}
                    {type === HEADER_TYPES.TD &&
                    <td key={index}
                        className={classNames(cell.sort ? "sort" : "", cell.className, this.state.sort && this.state.sort.column === cell.name ? "sorted" : "")}>
                        {cell.text}
                        {cell.sort && <button type="button" onClick={() => this.handleSort(cell.sort, cell.name)}>
                            <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/sort.svg")}></Isvg>
                        </button>}
                    </td>}
                </Fragment>
            )
        }) : null;

        const d = data.map(row => {
                const rowElement = Table.renderRow(row);
                const key = selectionOptions.getKey ? selectionOptions.getKey(row) : {id: row.id}; //in case no get key was defined, use id
                const disabled = row.isDisabled;
                const selected = selectionOptions && _.find(selectionOptions.selectedRows, key) !== undefined;
                return (
                    <CSSTransition classNames="tr" timeout={1000} key={JSON.stringify(key)}>
                        <tr key={JSON.stringify(key)}
                            onClick={() => selectionOptions && !disabled ? selectionOptions.selector(row) : ""}
                            className={classNames(rowElement.props.className, {selected, disabled})}>
                            {rowElement.props.children}
                        </tr>
                    </CSSTransition>)
            }
        );

        const classes = classNames("standard", {selectable: !!selectionOptions}, Table.className);

        return (
            <div className="selectable-table-wrapper">
                {headerElement && selectionOptions &&
                <table className={classNames(classes, "header", {fixed: fixedHeader})}>
                    <thead>
                    <tr>
                        {headerElement}
                    </tr>
                    </thead>
                </table>}
                <table className={classes}>
                    <TransitionGroup component="tbody">{d}</TransitionGroup>
                </table>
            </div>
        )
    }
};


export default class extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {filter: null};
    }

    setFilter(filter) {
        this.setState({filter});
    }

    render() {
        const {containerOptions: {singularTitle, pluralTitle}, Table, data, selectionOptions, fixedHeader, searchKeys} = this.props;
        const filterFunction = searchKeys ? defaultFilterWithKeys : defaultFilter;
        const filteredItems = this.state.filter ? data.filter(item => filterFunction(item, this.state.filter, searchKeys)) : data;

        return (
            <div className="searchable-selectable-table-container">
                <div className={classNames('widgets', {fixed: fixedHeader})}>
                    <ItemCountWidget className="float-left" singularTitle={singularTitle} pluralTitle={pluralTitle}
                                     itemCount={data.length} filteredItemCount={filteredItems.length}></ItemCountWidget>
                    <SelectedCountWidget count={selectionOptions.selectedRows.length}
                                         handleClear={selectionOptions.clearAll}/>
                    <SearchWidget className="float-right" items={data}
                                  setFilter={this.setFilter.bind(this)}></SearchWidget>
                </div>
                <SelectableTable Table={Table} data={filteredItems} selectionOptions={selectionOptions}
                                 fixedHeader={fixedHeader}/>
            </div>
        )
    }
}