import React from "react"
import {withRouter} from 'react-router-dom'
import InfoPopover from '../reactPopover/InfoPopover'
import Isvg from "react-inlinesvg";
import {connect} from "react-redux";
import _ from 'lodash';
import ErrorsPanel from '../errors/ErrorsPanel'
import classNames from 'classnames'
import errorsDispatcher from '../../dispatchers/errorsDispatcher'
import {DOCUMENTATION_URL, SHOW_TRIAL_INFO} from "../../utils/consts";
import {Query} from "react-apollo";
import {GET_ACCOUNT_INFO_QUERY} from "../../queries";


require("./footer.scss");

const calculateLicenseStatusFooter = (accountInfo) => {
    let licenseStatusText = 'LOADING...', warning = false, error = false;

    if (accountInfo) {
        const daysLeftForTrial = accountInfo.daysLeftForTrial;
        const trialEnded = daysLeftForTrial <= 0;
        const subscribed = accountInfo.subscriptionStatus === "subscribed";

        if (!trialEnded) {
            licenseStatusText = `FREE TRIAL [${daysLeftForTrial} ${daysLeftForTrial > 1 ? 'Days' : 'Day'} left]`;
            warning = daysLeftForTrial <= 7 && !subscribed;
        } else if (subscribed) {
            licenseStatusText = 'ACTIVE SUBSCRIPTION';
        } else {
            licenseStatusText = 'SUBSCRIPTION REQUIRED';
            error = true;
        }
    }
    return {licenseStatusText, warning, error};
};


const renderLicenseStatusFooter = ({licenseStatusText, warning, error}, history) => {
    return (
        <div className={classNames("license-status-footer", {"footer-error": error, "footer-warning": warning})}>
            <button onClick={() => {
                history.push('/license');
            }}>
                <Isvg cacheGetRequests={true} src={require("../../assets/svgs/exclamation-mark-triangle-icon.svg")}/>
                {licenseStatusText}
            </button>
        </div>
    );
};

const showTrialInfo = SHOW_TRIAL_INFO;

const Footer = (props) => {
    const {errors, errorsPanelOpen} = props;
    const shouldShowErrorsIcon = errors && errors.length > 0 && !errorsPanelOpen;
    const errorsMessage = shouldShowErrorsIcon ? (errors.length === 1 ? "1 Error" : `${errors.length} Errors`) : "";
    return (<Query query={GET_ACCOUNT_INFO_QUERY} fetchPolicy="network-only" pollInterval={30*1000}>
        {({loading, data}) => {
            if (loading) return <div/>;
            const {accountInfo} = data;
            const subscribed = accountInfo.subscriptionStatus === "subscribed";
            const pending = accountInfo.subscriptionStatus === "pending";
            return (
                <div>
                    <ErrorsPanel {...props}/>}
                    <footer className="footer">
                        <div className="footer-container">
                            {showTrialInfo && !subscribed && !pending && renderLicenseStatusFooter(calculateLicenseStatusFooter(accountInfo), props.history)}
                            {!showTrialInfo &&<InfoPopover textElement={<span>Controlled Availability</span>}
                                                           wrapperClassName={"controlled-availability"}
                                                           contentStyle={{width: "470px"}}
                                                           popoverText="This Controlled Availability release enables NetApp to gather feedback about Cloud Tiering from our customers. Production workloads are supported. You will not need to migrate your data when Cloud Tiering reaches General Availability."/>
                            }
                            <div className="documentation">
                                <a href={DOCUMENTATION_URL}
                                   target="_blank" rel="noopener noreferrer">
                                    <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/help.svg")}/>Documentation
                                </a>
                            </div>
                            <div className={classNames("errors", {"is-visible": shouldShowErrorsIcon})}
                                 onClick={() => props.toggleErrorsState()}>
                                <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/errors-footer.svg")}></Isvg>
                                {errorsMessage}
                            </div>
                        </div>
                    </footer>
                </div>
            );
        }}
    </Query>)
};

const mapStateToProps = state => {
    const errors = _.orderBy(_.values(state.errors.generalErrors.errorsMap), "time", "desc");
    return {
        errors,
        errorsPanelOpen: state.errors.generalErrors.errorsPanelOpen,
        errorsPanelSize: state.errors.generalErrors.errorsPanelSize
    }
};


export default withRouter(connect(mapStateToProps, errorsDispatcher)(Footer));