import React, {Component} from 'react';
import {connect} from 'react-redux';
import {Route, Switch} from 'react-router-dom';
import {Field, reduxForm} from 'redux-form';
import {InputField, SelectField} from "../forms/forms";
import stepinize from "./../wizard/stepinize";
import mapStateToProps from './wizardMapStateToProps'
import Validator from 'validatorjs';
import ButtonWithError from './../widgets/ButtonWithError';
import Isvg from "react-inlinesvg";
import {Query} from "react-apollo";
import {GET_IP_SPACES_QUERY} from "../../queries";
import _ from "lodash"
import LoadingIcon from '../widgets/LoadingIcon'
import classNames from "classnames";
import {calculateNextStep, getNextStepComponent} from "./WizardStarter";
import {DEFAULT_IP_SPACE} from "../../utils/consts"

require('./ipSpaceStep.scss');

const rules = {
    ipSpace: 'required'
};


const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({ipSpace: "IPspace"});
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

class IPSpaceStep extends Component {
    constructor(props, context) {
        super(props, context);
        const {wizard, change} = props;
        const ipSpace = _.get(wizard.selectedWizardParams, ["ipSpace"], DEFAULT_IP_SPACE);
        if (ipSpace) {
            this.state = {ipSpace};
            change("ipSpace", ipSpace);
        }
        else {
            this.state = {};
        }
    }



    submit(values) {
        const ipSpace = this.state.ipSpace || values.ipSpace;
        return Promise.resolve(true)
            .then(() => {
                const nextStep = calculateNextStep(this.props.wizard);
                this.props.wizard.nextStep(`/${ipSpace}/${nextStep}`, this.props.history);
            });

    }

    changeIPSpaceSelection(selectedOptions) {
        this.setState(Object.assign({}, this.state, {ipSpace: selectedOptions}));
    }

    handleError(error) {
        let errorMessage;
        if (error.networkError && error.networkError.result && error.networkError.result.errors)
            errorMessage = error.networkError.result.errors[0].message;
        else if (error.graphQLErrors)
            errorMessage = error.graphQLErrors[0].message;
        else console.log(`%c IPSpaceStep got GraphQL error ${JSON.stringify(error)}`, "font-weight:bold; color: red");
        if (errorMessage && this.state.error !== errorMessage) {
            this.setState({error: errorMessage});
        }
    }

    getIPSpaceUX(ipSpaceOptions) {
        const ipSpace = this.state.ipSpace;
        const selectedIPSpace = _.find(ipSpaceOptions, {value: ipSpace}) || undefined;
        const numOfOptions = ipSpaceOptions.length;
        if (numOfOptions > 1) {
            return (
                <Field component={SelectField} label="IPspace"
                       onChange={this.changeIPSpaceSelection.bind(this)}
                       valueField="value" name="ipSpace" options={ipSpaceOptions}
                       selectedValue={selectedIPSpace}
                       className="ip-space" noSearch={true}></Field>
            )
        }
        else {
            return (
                <div>
                    <Field name="ipSpace" component={InputField} label="IPspace"
                           type="text" disabled={true} value={selectedIPSpace? selectedIPSpace.label : ""}></Field>
                    {numOfOptions === 1 && <div className="ipspace-info">Only one suitable IPspace was found</div>}
                </div>
            )
        }

    }

    handleCompleteIPSpaceQuery(data) {
        const {change} = this.props;
        const ipSpaces = _.get(data, "ipSpaces", []);
        if (ipSpaces.length === 0){
            this.setState({error: "No suitable IPspaces were found, create one and try again"});
            this.props.change("ipSpace", "");
        }
        else {
            const defaultIpSpaceIndex = _.findIndex(ipSpaces, (ipSpace) => ipSpace === this.state.ipSpace);
            const defaultIpSpace = defaultIpSpaceIndex > 0 ? ipSpaces[defaultIpSpaceIndex] : ipSpaces[0];
            if (this.state.ipSpace !== defaultIpSpace) {
                this.setState({ipSpace: defaultIpSpace});
                change("ipSpace", defaultIpSpace);
            }
        }
    }

    render() {
        if (!this.props.wizard || !this.props.routingInfo || !this.props.routingInfo.urlParams || !this.props.routingInfo.urlParams.decodedParams.clusterId) {
            return <div>No Wizard or routing params yet...</div>
        }

        const {routingInfo: {urlParams: {decodedParams: {clusterId}}}, handleSubmit, error: propsError, up} = this.props;
        const stateError = this.state.error;
        return (
            <div className={classNames("ip-space-step wizard-step", {up, down: !up})}>
                <form onSubmit={handleSubmit(this.submit.bind(this))}>
                    <Query query={GET_IP_SPACES_QUERY} variables={{clusterId}} fetchPolicy="network-only"
                           onError={this.handleError.bind(this)} onCompleted={this.handleCompleteIPSpaceQuery.bind(this)}>
                        {({loading, error, data}) => {
                            if ((loading || !data) && !error) {//due to skip there might not be data yet
                                return (<LoadingIcon text={"IPspaces"} className="loading-ip-spaces" icon={<Isvg
                                    src={require('../../assets/svgs/ip-space-icon.svg')}/>}/>);
                            }
                            const ipSpaceOptions = _.map(_.get(data, 'ipSpaces', []), ipSpace => {
                                return {label: ipSpace, value: ipSpace};
                            });

                            return (
                                <div>
                                    <div className="step-title">
                                        <Isvg cacheGetRequests={true}
                                              src={require("./../../assets/svgs/ip-space-icon.svg")}/>
                                        <h3> Cluster Connectivity </h3>
                                        <div className="step-title-info">
                                            Select the IPspace on the cluster that you want to use to connect to your object storage provider.
                                        </div>
                                    </div>
                                    {!error && data && <div className="step-content">
                                        <div className="pane center">
                                            <div>
                                                {this.getIPSpaceUX(ipSpaceOptions)}
                                            </div>
                                        </div>
                                    </div>}
                                    <div className="wizard-button">
                                        <ButtonWithError className="button secondary" type="submit"
                                                         error={propsError || stateError}>
                                            Continue
                                        </ButtonWithError>
                                    </div>
                                </div>
                            )
                        }}
                    </Query>
                </form>
            </div>
        )
    }
}

const nextRoutes = ({match, wizard}) => {
    const nextStep = calculateNextStep(wizard, "Cluster Connectivity");
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:ipSpace/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    )
};

let IPSpaceStepForm = reduxForm({
    form: 'ipSpaceForm',
    validate
})(IPSpaceStep);

export default connect(mapStateToProps)(stepinize(IPSpaceStepForm, {
    name: () => `Cluster Connectivity`,
    nextRoutes,
}));
