import React, {Component} from 'react';
import {Mutation, Query} from 'react-apollo'
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import mapStateToProps from './wizardMapStateToProps'
import {ADD_TO_TIER, GET_CLUSTERS_QUERY, GET_REGIONS_QUERY, START_TIERING} from "../../queries";
import Isvg from "react-inlinesvg";
import ButtonWithError from "../widgets/ButtonWithError";
import localStorage from 'local-storage';
import _ from 'lodash'
import LoadingBar from '../widgets/LoadingBar';
import classNames from 'classnames'
import LoadingIcon from "../widgets/LoadingIcon";
import {STORAGE_CLASSES, DEFAULT_STORAGE_CLASS, TIERING_POLICY} from "../../utils/consts";

require('./reviewStep.scss');


const ReviewCard = ({className, cardTitle, values, icon}) => {
    return (
        <div className={`review-card ${className}`}>
            {/*<div className="overlay"></div>*/}
            <div className="card-title-container">
                <div className="icon">
                    {icon}
                </div>
                <div className="card-title">
                    {cardTitle}
                </div>
            </div>
            <div className="container">
                {values.map((value, index) => {
                    return (<div className="text" key={index}>
                        <div className="sub-title no-wrap">{value.subTitle}</div>
                        <div className="title no-wrap" title={value.title}>{value.title}</div>
                    </div>)
                })}
            </div>
        </div>
    )
};

class ReviewStep extends Component {
    constructor(props, context) {
        super(props, context);
        const {wizard: {wizardCluster}} = props;
        this.state = {tierStarted: false, wizardCluster};
    }

    componentDidMount() {
        this.setState({tierStarted: false});
    }

    continue() {
        this.props.history.push('/clusters');
    }

    handleError(error) {
        if (error.networkError && error.networkError.result && error.networkError.result.errors) this.setState({error: error.networkError.result.errors[0].message});
        else if (error.graphQLErrors) this.setState({error: error.graphQLErrors[0].message});
        else console.log(`%c ReviewStep got GraphQL error ${JSON.stringify(error)}`, "font-weight:bold; color: red");
    }

    handleSubmit(tieringMethod, mutationVariables, event) {
        this.setState({tierStarted: true});
        event.preventDefault();
        tieringMethod({
            variables: mutationVariables
        });
    }

    getDiagramUx({clusterName, numOfVolumes, totalVolumes, tieringPolicy, ipSpace}, {regionName, storageClass, bucketName, provider}) {
        const clusterValues = [
            {
                title: clusterName,
                subTitle: "Cluster Name"
            },
            {
                title: `${numOfVolumes}/${totalVolumes}`,
                subTitle: "Selected Volumes"
            },
            {
                title: `${tieringPolicy}`,
                subTitle: "Tiering Policy"
            }
        ];
        if (ipSpace) {
            clusterValues.push({title: `${ipSpace}`, subTitle: "IPspace"});
        }
        const bucketValues = [
            {
                title: storageClass,
                subTitle: "Storage Class"
            }
        ];

        if (regionName) {
            bucketValues.unshift({
                title: regionName,
                subTitle: "Region Name"
            })
        }

        if (bucketName) {
            bucketValues.unshift({
                title: bucketName,
                subTitle: "Bucket Name"
            })
        }

        return (<div className="diagram">
            <ReviewCard className="cluster" cardTitle="Cluster" values={clusterValues}
                        icon={<Isvg cacheGetRequests={true}
                                    src={require(`./../../assets/svgs/review-cluster.svg`)}/>}></ReviewCard>
            <div className="arrows"><Isvg src={require('./../../assets/svgs/review-arrows.svg')}/></div>

            <ReviewCard className="bucket" cardTitle={provider === "AWS" ? "S3 Bucket" : "Blob Object Storage"}
                        values={bucketValues} icon={<Isvg cacheGetRequests={true}
                                                          src={require(`./../../assets/svgs/review-bucket.svg`)}/>}></ReviewCard>
        </div>)

    }

    getMutationFormUX(handleSubmitMethod) {
        const error = this.state.error;
        return (
            <div>
                <form onSubmit={handleSubmitMethod}>
                    <div className="wizard-button">
                        <ButtonWithError type="submit" className="button secondary tier" error={error}>Tier
                            Cluster</ButtonWithError>
                    </div>
                </form>
            </div>
        )
    }

    handleErrorRegions(error) {
        let errorMessage;
        if (error.networkError && error.networkError.result && error.networkError.result.errors)
            errorMessage = error.networkError.result.errors[0].message;
        else if (error.graphQLErrors)
            errorMessage = error.graphQLErrors[0].message;
        else console.log(`%c BucketStep got GraphQL error ${JSON.stringify(error)}`, "font-weight:bold; color: red");
        if (errorMessage && this.state.error !== errorMessage) {
            this.setState({error: errorMessage});
        }
    }

    render() {
        if (!this.props.wizard || !this.props.wizard.selectedWizardParams || !this.props.routingInfo || !this.props.routingInfo.urlParams) {
            return <div></div> //sometimes it takes a bit time until all the url params are there...
        }
        const {up, wizard: {wizardType, wizardCluster, selectedWizardParams: {clusterId, region, storageClass, provider, tieringPolicy, ipSpace}}} = this.props;

        if (!tieringPolicy || !TIERING_POLICY[tieringPolicy]) { //sometimes it takes a bit time until all the url params are there...
            return <div></div>
        }
        const volumes = _.compact(localStorage.get("selectedVolumes"));
        const awsKeys = localStorage.get("selectedAwsKeys");
        const {volumes: {eligibleVolumes}} = wizardCluster;
        const selectedClusterName = wizardCluster.name;
        const tieringPolicyName = TIERING_POLICY[tieringPolicy].name;
        return (
            <div className={classNames("review-step wizard-step", {
                up,
                down: !up,
                "tier-started": this.state.tierStarted
            })}>
                {/*<div className="background"></div>*/}
                <div className="step-title">
                    <Isvg src={require("./../../assets/svgs/review-icon.svg")}/>
                    <h3>Review & Approve</h3>
                </div>
                {wizardType === "tier" &&
                <Query query={GET_REGIONS_QUERY}
                       variables={{type: provider, clusterId: wizardCluster.clusterId, regionCode: region}}
                       skip={!provider}
                       onError={this.handleErrorRegions.bind(this)}>
                    {({loading, error, data}) => {
                        if (loading || !data) { //due to skip there might not be data yet
                            return (<LoadingIcon text={"Regions"} className="loading-regions" icon={<Isvg
                                src={require('../../assets/svgs/bucket-icon.svg')}/>}/>);
                        }
                        if (error) return `Error! ${error.message}`;
                        const regionName = region ? _.find(data.regions, {region: region}).regionName : "";
                        const storageClassName = storageClass ? _.find(STORAGE_CLASSES[provider], {storageClass}).storageClassName : STORAGE_CLASSES[provider][DEFAULT_STORAGE_CLASS].storageClass;
                        return this.getDiagramUx({
                                clusterName: selectedClusterName,
                                numOfVolumes: volumes.length,
                                totalVolumes: eligibleVolumes,
                                tieringPolicy: tieringPolicyName,
                                ipSpace
                            },
                            {
                                regionName,
                                storageClass: storageClassName,
                                provider
                            });
                    }}
                </Query>}
                {wizardType === "tier" && <Mutation mutation={START_TIERING} onCompleted={this.continue.bind(this)}
                                                    onError={this.handleError.bind(this)} errorPolicy="all"
                                                    refetchQueries={[
                                                        {query: GET_CLUSTERS_QUERY, awaitRefetchQueries: false}
                                                    ]}>
                    {(startTiering, {loading}) => {
                        if (loading) {
                            return <LoadingBar/>;
                        }
                        const variables = {
                            clusterId,
                            volumeIdentifiers: volumes,
                            providerProperties: {
                                cloudProvider: provider,
                                storageClass,
                                region,
                                awsKeys: provider === "AWS" ? awsKeys : undefined
                            },
                            tieringPolicy,
                            ipSpace
                        };
                        return this.getMutationFormUX(this.handleSubmit.bind(this, startTiering, variables));
                    }}
                </Mutation>}
                {wizardType === "retier" &&
                <Query query={GET_CLUSTERS_QUERY} variables={{clusterId}} skip={!clusterId} fetchPolicy="network-only">
                    {({loading, error, data}) => {
                        if (loading || !data) { //due to skip there might not be data yet
                            return (<LoadingIcon text={"Cluster"} className="loading-cluster" icon={<Isvg
                                src={require('../../assets/svgs/review-cluster.svg')}/>}/>);
                        }
                        if (error) return `Error! ${error.message}`;
                        const {clusters} = data;
                        const cluster = clusters[0];
                        const {tierInformation: {regionName, bucketName, tierLevel}, agentCloudProvider} = cluster;
                        return this.getDiagramUx({
                                clusterName: cluster.name,
                                numOfVolumes: volumes.length,
                                totalVolumes: eligibleVolumes,
                                tieringPolicy: tieringPolicyName
                            },
                            {
                                regionName,
                                storageClass: tierLevel,
                                bucketName,
                                provider: agentCloudProvider
                            });
                    }}

                </Query>}
                {wizardType === "retier" && <Mutation mutation={ADD_TO_TIER} onCompleted={this.continue.bind(this)}
                                                      onError={this.handleError.bind(this)} errorPolicy="all"
                                                      refetchQueries={[
                                                          {query: GET_CLUSTERS_QUERY, awaitRefetchQueries: false}
                                                      ]}>
                    {(addToTier, {loading}) => {
                        if (loading) {
                            return <LoadingBar/>;
                        }
                        const variables = {
                            clusterId,
                            volumeIdentifiers: volumes,
                            tieringPolicy
                        };
                        return this.getMutationFormUX(this.handleSubmit.bind(this, addToTier, variables));
                    }}
                </Mutation>}
            </div>
        )
    }
}

export default connect(mapStateToProps)(stepinize(ReviewStep, {
    name: () => `Review`,
}));
