import React, {Component} from 'react';
import {connect} from 'react-redux';
import {Route, Switch} from 'react-router-dom';
import stepinize from "./../wizard/stepinize";
import mapStateToProps from './wizardMapStateToProps'
import {Query} from "react-apollo";
import Isvg from 'react-inlinesvg';
import {GET_VOLUMES_QUERY} from "../../queries";
import SearchableSelectableTable, {HEADER_TYPES} from '../table/SearchableSelectableTable'
import _ from 'lodash'
import ButtonWithError from "../widgets/ButtonWithError";
import {reduxForm, stopSubmit, SubmissionError} from "redux-form";
import localStorage from 'local-storage';
import LoadingIcon from '../widgets/LoadingIcon'
import classNames from 'classnames'
import {buildCapacityObject, extractONTAPVersion} from "../../utils/utils";
import InfoPopover from "../reactPopover/InfoPopover"
import {routesObject} from "../../actions";
import LearnMoreDialog from './LearnMoreDialog'
import TieringPolicyDialog from "./TieringPolicyDialog"
import {openDialog} from "../dialog/Dialog";
import {TIERING_POLICY} from "../../utils/consts";
import {calculateNextStep, getNextStepComponent} from "./WizardStarter";

require("./sourceVolumesStep.scss");


class SourceVolumesStep extends Component {
    constructor(props, context) {
        super(props, context);
        const selectedVolumes = (_.compact(localStorage.get("selectedVolumes")) || []);
        const volumes = _.map(this.props.volumes, volume => {
            const capacityObject = buildCapacityObject(_.get(volume, "capacity", 0));
            const usedPerformanceAndTieredCapacityObject = buildCapacityObject(_.get(volume, "usedPerformanceAndTieredCapacity", 0));
            const inactiveCapacityObject = buildCapacityObject(_.get(volume, "inactiveCapacity",0));
            const propsVolume = {
                capacityText: `${capacityObject.value} ${capacityObject.unit}`,
                usedPerformanceAndTieredCapacityText: `${usedPerformanceAndTieredCapacityObject.value} ${usedPerformanceAndTieredCapacityObject.unit}`,
                inactiveCapacityText: `${inactiveCapacityObject.value} ${inactiveCapacityObject.unit}`,
                inactiveCapacityPercentText: `${volume.inactiveCapacityPercent} %`,
                ...volume
            };
            if (volume.tieringPolicy !== "none" || !volume.isEligibleForTiering) return {
                isDisabled: true,
                ...propsVolume
            };
            return propsVolume;
        });
        // console.log(`%c Volumes from query with props:\n${JSON.stringify(volumes)}`, "font-weight: bold; color: green");

        const tieredVolumes = this.getAllTieredVolumes(volumes);
        const notEligible = this.getNotEligibleForTieringVolumes(volumes);
        const available = this.getAvailableVolumes(volumes);

        const allVolumesCount = volumes.length;
        const notEligibleCount = notEligible.length;
        const tieredCount = tieredVolumes.length;
        const availableCount = allVolumesCount - notEligibleCount - tieredCount;
        const isColdDataEstimated = volumes && volumes.length > 0 ? volumes[0].isEstimatedInactiveCapacity : false;

        const tierStatusFilters = [
            {name: "available", text: "Available for Tiering", count: availableCount, checked: true},
            {name: "tiered", text: "Tiered Volumes", count: tieredCount, checked: true},
            {name: "unavailable", text: "Unavailable for Tiering", count: notEligibleCount, checked: true}
        ];

        const ontapVersion = extractONTAPVersion(props.wizard.wizardCluster.ontapClusterProperties.ontapVersion);

        this.state = {
            tieringPolicy: ontapVersion < 9.4 ? TIERING_POLICY.snapshot_only : TIERING_POLICY.auto,
            isColdDataEstimated,
            selectedVolumes,
            allSelected: false,
            tiered: tieredVolumes,
            unavailable: notEligible,
            available,
            tierStatusFilters,
            showVolumes: volumes,
            ontapVersion
        };
    }

    getAllTieredVolumes(volumes) {
        return _.compact(_.map(volumes, volume => {
            if (volume.tieringPolicy !== "none")
                return volume;
            return null;
        }));
    }

    getNotEligibleForTieringVolumes(volumes) {
        return _.compact(_.map(volumes, volume => {
            if (!volume.isEligibleForTiering)
                return volume;
            return null;
        }));

    }

    getAvailableVolumes(volumes) {
        return _.compact(_.map(volumes, volume => {
            if (volume.isEligibleForTiering && volume.tieringPolicy === "none")
                return volume;
            return null;
        }));
    }

    submit(allVolumes) {
        if (this.state.selectedVolumes.length === 0) {
            const error = {_error: `In order to continue to the next step please select at least one volume to tier.`};
            throw new SubmissionError(error);
        }

        return Promise.resolve(true)
            .then(() => {
                const selectedVolumes = this.state.selectedVolumes;
                localStorage.set("selectedVolumes", selectedVolumes);
                const nextStep = calculateNextStep(this.props.wizard);
                this.props.wizard.nextStep(`/${this.state.tieringPolicy.type}/${nextStep}`, this.props.history);
            });
    }

    applyFilter(filters, sort) {
        const updatedTierStatusFilters = this.state.tierStatusFilters.map(value => {
            const filter = _.find(filters, {name: value.name});
            value.checked = filter.checked;
            return value;
        });
        const showVolumes = _.reduce(filters, (volumes, filter) => {
                if (filter.checked) {
                    return [...volumes, ...this.state[filter.name]];
                }
                else return volumes;
        }, []);
        if (!sort){
            this.setState({tierStatusFilter: updatedTierStatusFilters, showVolumes});
        }
        else {
            this.setState({tierStatusFilter: updatedTierStatusFilters}); //the sort will update the volumes
            this.applySort(sort.column, sort.ascending, showVolumes);
        }
    }

    applySort(column, ascending, volumes) {
        const sortedVolumes = _.sortBy(volumes ? volumes : this.state.showVolumes, [column]);
        this.setState({showVolumes: ascending ? sortedVolumes : _.reverse(sortedVolumes)})
    }

    getTableHeader() {
        const sort = this.applySort.bind(this);
        return [
            {className: "check", type: HEADER_TYPES.ALL, allOptions: {
                    allSelected: this.state.allSelected,
                    toggleAll: this.toggleAll.bind(this, this.state.showVolumes)
                }},
            {name: "name", text: "Volume Name", sort},
            {name: "svmName", text: "SVM Name", sort},
            {name: "capacity", text: "Volume Size", sort},
            {name: "usedPerformanceAndTieredCapacity", text: "Used Size", sort},
            {name: "inactiveCapacity", text: this.state.isColdDataEstimated ? "Cold Data (Estimated)" : "Cold Data", sort},
            {text: "Tier Status", className: "tier-status", type: HEADER_TYPES.FILTER, filterOptions:{
                    filterName: "tierStatus",
                    filter: this.applyFilter.bind(this),
                    filterValues: this.state.tierStatusFilters
                },
                sort //must give the sort method to allow sorting after the filter is applied
            }
        ]
    }

    renderRow(volume) {
        return (
            <div className="table-rows ">
                <td className="check"><div><Isvg cacheGetRequests={true}
                                            src={require("./../../assets/svgs/checkmark-fill-circle-icon.svg")}></Isvg></div>
                </td>
                <td className="volume-name">
                    <div title={volume.name} className="no-wrap">{volume.name}</div>
                </td>
                <td className="svm-name no-wrap" title={volume.svmName}>{volume.svmName}</td>
                <td className="volume-size">{volume.capacityText}</td>
                <td className="used-size">{volume.usedPerformanceAndTieredCapacityText}</td>
                <td className="cold-data"><span>{volume.inactiveCapacityText}</span><span>{volume.inactiveCapacityPercentText}</span></td>
                {volume.isDisabled && <td className={classNames("info",
                    {"not-eligible" : !volume.isEligibleForTiering, "tiered" : volume.isDisabled && volume.isEligibleForTiering})}>
                    <div>
                        {!volume.isEligibleForTiering && <InfoPopover contentStyle={{width: "360px"}} textElement={<span className="italic not-eligible">Unavailable for Tiering</span>} popoverClassName="not-eligible" wrapperClassName="not-eligible"
                                     popoverText={volume.tieringRestrictionReason}/>}
                        {volume.isEligibleForTiering && <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/checkmark-fill-circle-icon.svg")}></Isvg>}
                        {volume.isEligibleForTiering ? "Tiered Volume" : ""}
                    </div>
                </td>}
                {!volume.isDisabled && <td className="info">Available for Tiering</td>}
            </div>
        )
    }

    handleSelect(selectedVolume) {
        this.props.dispatch(stopSubmit("volumes", {})) //clear errors
        const selectedVolumeKey = this.getKey(selectedVolume);
        const found = _.find(this.state.selectedVolumes, selectedVolumeKey);
        if (found) {
            const selectedVolumes = _.reject(this.state.selectedVolumes, selectedVolumeKey);
            this.setState(Object.assign({}, this.state, {selectedVolumes: selectedVolumes, allSelected: false}));
        } else {
            this.setState(Object.assign({}, this.state, {selectedVolumes: _.concat(this.state.selectedVolumes, selectedVolumeKey)}));
        }
    }

    toggleAll(volumes) {
        this.props.dispatch(stopSubmit("volumes", {})); //clear errors
        if (this.state.allSelected) //unselect all besides those already tiered
            this.setState({selectedVolumes: []});
        else {
            this.setState({
                selectedVolumes: _.compact(_.map(volumes, volume => {
                    if (volume.tieringPolicy !== "none" || !volume.isEligibleForTiering) return null;
                    else return this.getKey(volume)
                }))
            });
        }
        this.setState({allSelected: !this.state.allSelected});
    }

    clearAll() {
        this.setState({selectedVolumes: [], allSelected: false});
    }

    getKey(volume) {
        return _.pick(volume, ["name", "svmName"]);
    }

    applyTieringPolicy(tieringPolicy) {
        this.setState({tieringPolicy})
    }

    render() {
        const {error, handleSubmit, up} = this.props;
        const {isColdDataEstimated} = this.state;
        return (
            <div className={classNames("volumes-step wizard-step", {up, down: !up})}>
                <div>
                    <button type="button" className="learn-more-button"
                            onClick={(el) => {openDialog(<LearnMoreDialog className="learn-more-dialog"  estimated={isColdDataEstimated}></LearnMoreDialog>, el.target)}}>{"Learn more about these volumes >"}</button>
                    <div className="step-title">
                        <Isvg src={require("./../../assets/svgs/source-volumes-icon.svg")}/>
                        <h3>Select Source Volumes</h3>
                        <div className="tiering-policy">All tiered volumes will receive Tieirng Policy
                            <button type="button" className="tiering-policy-button"
                                    onClick={(el) => {openDialog(<TieringPolicyDialog className="tiering-policy-dialog confirm" ontapVersion={this.state.ontapVersion} apply={this.applyTieringPolicy.bind(this)} tieringPolicy={this.state.tieringPolicy}>
                                    </TieringPolicyDialog>, el.target)}}>{` ${this.state.tieringPolicy.name} >`}</button>
                        </div>
                    </div>
                    <form onSubmit={handleSubmit(this.submit.bind(this, this.state.showVolumes))} className="volumes-form">

                        <div className="volumes">
                            <SearchableSelectableTable
                                fixedHeader={true}
                                Table={{
                                    getTableHeader: this.getTableHeader.bind(this),
                                    renderRow: this.renderRow.bind(this)
                                }}
                                data={this.state.showVolumes}
                                searchKeys={["name", "svmName", "capacityText", "usedPerformanceAndTieredCapacityText", "inactiveCapacityText", "inactiveCapacityPercentText"]}
                                selectionOptions={{
                                    selector: this.handleSelect.bind(this),
                                    selectedRows: this.state.selectedVolumes,
                                    clearAll: this.clearAll.bind(this),
                                    getKey: this.getKey.bind(this)
                                }}
                                containerOptions={{
                                    singularTitle: "Volume",
                                    pluralTitle: "Volumes",
                                }}>
                            </SearchableSelectableTable>
                        </div>
                        <div className="wizard-button">
                            <ButtonWithError className="button secondary" type="submit" error={error}>
                                Continue
                            </ButtonWithError>
                        </div>
                    </form>
                </div>
            </div>
        )
    }
}

const nextRoutes = ({match, wizard}) => {
    const nextStep = calculateNextStep(wizard, "Source Volumes");
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:tieringPolicy/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    )
};

const SourceVolumesStepForm = reduxForm({
    form: 'volumes'
})(SourceVolumesStep);


const ConnectedSourceVolumesStep = connect(mapStateToProps)(stepinize(SourceVolumesStepForm, {
    name: () => `Source Volumes`,
    nextRoutes
}));

export default (props) => {
    const clusterId = props.match.params.clusterId;

    return (
        <Query query={GET_VOLUMES_QUERY} variables={{clusterId}} fetchPolicy="network-only">
            {({loading, error, data}) => {
                if (loading)
                    return (<LoadingIcon text={"Volumes"} className="loading-volumes"
                                         icon={<Isvg src={require('../../assets/svgs/source-volumes-icon.svg')}/>}/>);

                if (error) {
                    routesObject.routes.push("/");
                    return <div></div>;
                } //in case of an error, return to dashboard
                return (
                    <ConnectedSourceVolumesStep volumes={data.volumes} {...props}/>
                );
            }}
        </Query>
    )
}


